/*
 * FILE:
 *   enet_basic_socket.cpp
 *
 * AUTHOR:
 *   Stephen Thompson, 2008
 *
 * COPYRIGHT:
 *   Public Domain
 *
 */

#include "enet_basic_socket.hpp"

namespace Coercri {

    EnetBasicSocket::EnetBasicSocket(int p)
        : port(p)
    {
        ENetAddress addr;
        addr.host = ENET_HOST_ANY;
        addr.port = port;
        sock = enet_socket_create(ENET_SOCKET_TYPE_DATAGRAM, &addr);
        enet_socket_set_option(sock, ENET_SOCKOPT_NONBLOCK, 1);
        enet_socket_set_option(sock, ENET_SOCKOPT_BROADCAST, 1);
    }

    EnetBasicSocket::~EnetBasicSocket()
    {
        enet_socket_destroy(sock);
    }

    bool EnetBasicSocket::receive(std::string &address, std::string &msg)
    {
        ENetAddress addr;

        char mybuf[2048];
        ENetBuffer buffer;
        buffer.data = mybuf;
        buffer.dataLength = sizeof(mybuf);
        
        int num_bytes = enet_socket_receive(sock, &addr, &buffer, 1);

        if (num_bytes <= 0) {
            return false;
        } else {
            msg.assign(mybuf, num_bytes);
            enet_address_get_host_ip(&addr, mybuf, sizeof(mybuf));
            address = mybuf;
            return true;
        }
    }

    void EnetBasicSocket::send(const std::string &address, const std::string &msg)
    {
        ENetAddress addr;
        enet_address_set_host(&addr, address.c_str());
        addr.port = port;

        ENetBuffer buffer;
        buffer.data = const_cast<char*>(msg.data());
        buffer.dataLength = msg.length();

        enet_socket_send(sock, &addr, &buffer, 1);
    }
    
    void EnetBasicSocket::broadcast(const std::string &msg)
    {
        ENetAddress addr;
        addr.host = ENET_HOST_BROADCAST;
        addr.port = port;

        ENetBuffer buffer;
        buffer.data = const_cast<char*>(msg.data());
        buffer.dataLength = msg.length();
        
        enet_socket_send(sock, &addr, &buffer, 1);
    }    
}
