/*
 * colour_change.cpp
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "colour_change.hpp"

#include <algorithm>
using namespace std;

void ColourChange::add(Coercri::Color old_col, Coercri::Color new_col)
{
    // NB would be faster to do some kind of incremental sort (eg an insertion sort),
    // but it's easier just to append the new value and then re-sort the whole lot :)
    mappings.push_back(make_pair(old_col, new_col));
    stable_sort(mappings.begin(), mappings.end());
}

struct CmpFirst {
    bool operator()(const pair<Coercri::Color,Coercri::Color> &lhs,
                    const pair<Coercri::Color,Coercri::Color> &rhs) const
    {
        return lhs.first < rhs.first;
    }
};

bool ColourChange::lookup(Coercri::Color old_col, Coercri::Color &new_col) const
{
    pair<Coercri::Color, Coercri::Color> x;
    x.first = old_col;

    CmpFirst cmp;
    vector<pair<Coercri::Color,Coercri::Color> >::const_iterator it
        = lower_bound(mappings.begin(), mappings.end(), x, cmp);
    if (it == mappings.end() || cmp(x, *it)) {
        return false;
    } else {
        new_col = it->second;
        return true;
    }
}

//
// serialization
//

void ColourChange::serialize(Coercri::NetworkConnection &remote) const
{
    remote.putCInt(mappings.size());
    for (int i=0; i<mappings.size(); ++i) {
        remote.putUbyte(mappings[i].first.r);
        remote.putUbyte(mappings[i].first.g);
        remote.putUbyte(mappings[i].first.b);
        remote.putUbyte(mappings[i].first.a);
        remote.putUbyte(mappings[i].second.r);
        remote.putUbyte(mappings[i].second.g);
        remote.putUbyte(mappings[i].second.b);
        remote.putUbyte(mappings[i].second.a);
    }
}

ColourChange::ColourChange(Coercri::NetworkConnection &remote)
{
    mappings.resize(remote.getCInt());
    for (int i=0; i<mappings.size(); ++i) {
        mappings[i].first.r = remote.getUbyte();
        mappings[i].first.g = remote.getUbyte();
        mappings[i].first.b = remote.getUbyte();
        mappings[i].first.a = remote.getUbyte();
        mappings[i].second.r = remote.getUbyte();
        mappings[i].second.g = remote.getUbyte();
        mappings[i].second.b = remote.getUbyte();
        mappings[i].second.a = remote.getUbyte();
    }
}
