/*
 * graphic_transform.cpp
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "colour_change.hpp"
#include "gfx_resizer.hpp"
#include "graphic_transform.hpp"
#include "round.hpp"

shared_ptr<const Coercri::PixelArray> CreateGraphicWithCC(shared_ptr<const Coercri::PixelArray> pixels,
                                                          const ColourChange &cc)
{
    const int width = pixels->getWidth();
    const int height = pixels->getHeight();
    shared_ptr<Coercri::PixelArray> new_pixels(new Coercri::PixelArray(width, height));

    for (int y = 0; y < height; ++y) {
        for (int x = 0; x < width; ++x) {
            const Coercri::Color &in = (*pixels)(x,y);
            Coercri::Color &out = (*new_pixels)(x,y);
            if (!cc.lookup(in, out)) {
                out = in;
            }
        }
    }

    return new_pixels;
}

shared_ptr<const Coercri::PixelArray> CreateResizedGraphic(const GfxResizer &resizer,
                                                           shared_ptr<const Coercri::PixelArray> original,
                                                           int new_width, int new_height,
                                                           int old_hx, int old_hy, int &new_hx, int &new_hy)
{
    const int old_width = original->getWidth();
    const float scale = float(new_width) / float(old_width);
    new_hx = Round(float(old_hx) * scale);
    new_hy = Round(float(old_hy) * scale);
    return resizer.resize(original, new_width, new_height);
}
