/*
 * mini_map.hpp
 *
 * Mini-Map class
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef MINI_MAP_HPP
#define MINI_MAP_HPP

#include "mini_map_colour.hpp"

#include "coercri/gfx/gfx_context.hpp"

#include <map>
#include <vector>
using namespace std;

class GfxManager;
class MapCoord;

class MiniMap {
public:
    MiniMap() : width(0), height(0) { }
    void create(int w, int h);

    // Accessors
    int getWidth() const { return width; }
    int getHeight() const { return height; }
    MiniMapColour getColour(const MapCoord &mc) const; // useful for LocDview::isSqMapped.
    
    // Modifiers
    void setHighlight(const MapCoord &mc, int id);  // add a 'flashing' square. 
    void setColour(const MapCoord &mc, MiniMapColour col);
    void wipe();       // sets all tile colours to zero (NB: does not affect highlights.)

    // Draw the map.
    // t = time (used to do the flashing indicators).
    void draw(Coercri::GfxContext &gc, int left, int top, int width, int height, int t) const;
    
private:
    struct Highlighter {        
        Highlighter(const MiniMap &m, int t);  // sets highlights to appropriate colour
        ~Highlighter();  // resets highlights to their original colours.
        const MiniMap &mini_map;
    };
    friend struct Highlighter;

    int width, height;
    mutable vector<MiniMapColour> data; // this must be mutable for Highlighter to work properly.
    struct Highlight {
        MapCoord pos;
        MiniMapColour old_colour;
    };
    map<int, Highlight> highlights;
};

#endif
