/*
 * sound_manager.cpp
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "rstream/rstream.hpp"
#include "sound_manager.hpp"

void SoundManager::loadSound(const string &name)
{
    if (!sound_driver) return;
    
    if (sound_map.find(name) == sound_map.end()) {
        shared_ptr<RStream> str(new RStream(name));
        sound_map.insert(make_pair(name, sound_driver->loadSound(str)));
    }
}

void SoundManager::playSound(const string &name, int frequency)
{
    if (!sound_driver) return;
    
    map<string, shared_ptr<Coercri::Sound> >::iterator it = sound_map.find(name);
    if (it == sound_map.end()) {
        loadSound(name);
        it = sound_map.find(name);
        ASSERT(it != sound_map.end());
    }
    sound_driver->playSound(it->second, frequency);
}

void SoundManager::clear()
{
    sound_map.clear();
}
